@@ -14,19 +14,20 @@ as provided by exporting UCSC gene data in GTF format.
 # A package-unique, descriptive module name used for filenames, etc
 MODULE="feature_aggregation"
 
-__version__ = "$Revision: 243 $"
+__version__ = "$Revision: 280 $"
 
 import os
 import sys
 
 from collections import defaultdict
 from functools import partial
-from numpy import arange, array, linspace, round
-
-from . import Annotations, log, Segmentation, \
-    DTYPE_SEGMENT_START, DTYPE_SEGMENT_END, DTYPE_SEGMENT_KEY, DTYPE_STRAND
-from .common import die, fill_array, get_ordered_labels, make_tabfilename, map_segment_label, r_plot, r_source, setup_directory, tab_saver
+from numpy import arange, array, empty, linspace, round
 
+from . import (Annotations, log, Segmentation, DTYPE_SEGMENT_START,
+               DTYPE_SEGMENT_END, DTYPE_SEGMENT_KEY, DTYPE_STRAND)
+from .common import (die, fill_array, get_ordered_labels, make_tabfilename,
+                     map_segment_label, r_plot, r_source, setup_directory,
+                     tab_saver)
 from .html import save_html_div
 
 NAMEBASE = "%s" % MODULE
@@ -37,6 +38,7 @@ STATIC_FIELDNAMES = ["group", "component", "offset"]
 
 HTML_TITLE_BASE = "Feature aggregation"
 
+# REGION_MODE, GENE_MODE mean do quantile bins
 POINT_MODE = "point"
 REGION_MODE = "region"
 GENE_MODE = "gene"
@@ -401,25 +403,24 @@ class GeneAnnotations(Annotations):
         """
 
         def get_transcript_length(rows):
-            assert len(rows) > 0
-            start = rows[0]['start']
-            end = rows[0]['end']
-            for row in rows[1:]:
-                start = min(start, row['start'])
-                end = max(end, row['end'])
+            start = min(row['start'] for row in rows)
+            end = max(row['end'] for row in rows)
 
             return end - start
 
+        # XXX: this could probably be simplified vastly using max()
+        # and a generator expression; the semantics would change
+        # slightly in case of ties; you would get the transcript with
+        # the highest ID instead of the first one of that length.
+        # check to see if this is OK
         max_length = 0
-        longest = None
+        res = None
         for id, rows in transcript_dict.iteritems():
-            start = None
-            end = None
             length = get_transcript_length(rows)
             if length > max_length:
                 max_length = length
-                longest = (id, rows)
-        return longest
+                res = (id, rows)
+        return res
 
 
 def start_R():
@@ -431,19 +432,39 @@ def print_bed_from_gene_component(features, component="terminal exon"):
         for chrom, chrom_features in features.iteritems():
             for feature in chrom_features:
                 if feature["name"].startswith(component):
-                    print >> ofp, "%s\t%s\t%s\t%s" % (chrom,
-                                                      feature["start"],
-                                                      feature["end"],
-                                                      feature["name"])
+                    print >>ofp, "%s\t%s\t%s\t%s" % (chrom,
+                                                     feature["start"],
+                                                     feature["end"],
+                                                     feature["name"])
+
+def calc_bin_starts(start, end, num_bins):
+    """Calculate internal bin locations"""
+    if num_bins <= 0:
+        return array([], dtype="int")
+
+    # linspace: Return evenly spaced numbers over a specified interval.
+    # endpoint=False
+    bin_starts_unrounded = linspace(start, end, num_bins, False)
+
+    res = empty(bin_starts_unrounded.shape, dtype="int")
+    round(bin_starts_unrounded, out=res)
+    return res
 
 def calc_feature_windows(feature, labels, mode, component_bins):
-    """Return a list of tuples: (component, window (array of base indices))"""
+    """Calculate where the bin windows for this feature begin; Spread
+    internal bins throughout feature
+
+    Return a list of tuples: (component, window)
+
+    component: str (usually from *_COMPONENT/*_COMPONENTS constants)
+    window: numpy.ndarray(dtype=int) of base indices
+    """
     name = labels[feature['key']]
     start = feature['start']
     end = feature['end']
     try:
         strand = feature['strand']
-    except IndexError, KeyError:
+    except (IndexError, KeyError):
         assert mode != GENE_MODE
         strand = "."
 
@@ -453,6 +474,8 @@ def calc_feature_windows(feature, labels, mode, component_bins):
     # Include flanks by default
     num_5p_bins = component_bins[FLANK_5P]
     num_3p_bins = component_bins[FLANK_3P]
+
+    # remove flanks if gene mode and not on one end or other
     if mode == GENE_MODE:
         assert name in component_bins
         component = name
@@ -463,7 +486,6 @@ def calc_feature_windows(feature, labels, mode, component_bins):
         if component != TERMINAL_EXON:
             # Not terminal exon, so no 3' flank
             num_3p_bins = 0
-
     elif mode == REGION_MODE:
         component = REGION_COMPONENT
     else:
@@ -472,24 +494,19 @@ def calc_feature_windows(feature, labels, mode, component_bins):
         num_internal_bins = 0
 
     if component is not None:
-        try:
-            num_internal_bins = component_bins[component]
-        except KeyError:
-            num_internal_bins = 0
+        # default value of 0
+        num_internal_bins = component_bins.get(component, 0)
 
     if num_internal_bins > length:
+        # don't calculate internal bins in this case
         #print >> sys.stderr, "Warning: %d %s bins > %d bases" % \
         #    (num_internal_bins, component, length)
         num_internal_bins = 0
 
-    if num_internal_bins > 0:
-        # Calculate internal bin locations (end included in linspace)
-        internal_bins = round(linspace(start, end - 1,
-                                       num_internal_bins)).astype("int")
-    else:
-        internal_bins = []
+    internal_bins = calc_bin_starts(start, end, num_internal_bins)
 
     # Calculate flanking base locations
+    # XXX: what happens when the flanking regions are off the chromosome?
     if strand == "-":
         bins_5p = arange(end, end + num_5p_bins, dtype="int")[::-1]
         internal_bins = internal_bins[::-1]  # reverse
@@ -498,20 +515,19 @@ def calc_feature_windows(feature, labels, mode, component_bins):
         bins_5p = arange(start - num_5p_bins, start, dtype="int")
         bins_3p = arange(end, end + num_3p_bins, dtype="int")
 
-    windows = []
+    res = []
     if len(bins_5p) > 0:
-        windows.append((FLANK_5P, bins_5p))
+        res.append((FLANK_5P, bins_5p))
     if len(internal_bins) > 0:
-        windows.append((component, internal_bins))
+        res.append((component, internal_bins))
     if len(bins_3p) > 0:
-        windows.append((FLANK_3P, bins_3p))
+        res.append((FLANK_3P, bins_3p))
 
     #for key, val in locals().iteritems():
     #    print >>sys.stderr, "%s : %s" % (key, val)
     #sys.exit(1)
 
-    return windows
-
+    return res
 
 ## Accepts data from dict: chr -> dict {"start", "end", "strand"})
 ##   zero-based start and end (exclusive) indices
@@ -543,11 +559,12 @@ def calc_aggregation(segmentation, features, mode, groups, components,
             log("\t\t", end="")
             log("%s; %d bins" % (component, component_bins[component]))
 
+    # counts is an accumulator
     # dict:
     #   key: feature_group
     #   value: dict:
     #      key: component_name
-    #      value: numpy.array histogram [bin, label_key]
+    #      value: numpy.ndarray histogram [bin, label_key]
     counts = dict([(group,
                     dict([(component, fill_array(0, (bins, len(labels))))
                           for component, bins in component_bins.iteritems()]))
@@ -571,8 +588,8 @@ def calc_aggregation(segmentation, features, mode, groups, components,
         segmentation_end = segments['end'][-1]
 
         # Map entire chromosome's segments into an array of label_keys
-        # XXXopt: THIS LINE TAKES UP TO 3.2 GB OF MEMORY!!!!
-        segment_map, sentinal = map_segment_label(
+        # XXXopt: this line takes up to 3.2 GB of memory
+        segment_map, sentinel = map_segment_label(
             segments, (segmentation_start, segmentation_end))
 
         # For each feature, tally segments in window
@@ -586,14 +603,16 @@ def calc_aggregation(segmentation, features, mode, groups, components,
                 group = groups[0]
 
             group_counts = counts[group]
+            # dict:
+            #      key: component_name
+            #      value: numpy.ndarray histogram [bin, label_key]
 
-            # Spread internal bins throughout feature
+            # Spread internal bins throughout feature if not POINT_MODE
             component_windows = calc_feature_windows(feature, features.labels,
                                                      mode, component_bins)
 
             # Scan window, tallying segments observed
             for component, window in component_windows:
-
                 component_counts = group_counts[component]
                 #log("component: %r" % component)
                 #log("component_counts.shape: %r" % str(component_counts.shape))
@@ -605,13 +624,15 @@ def calc_aggregation(segmentation, features, mode, groups, components,
                 keep = ((window >= segmentation_start) &
                         (window < segmentation_end))
                 label_keys = segment_map[map_indices[keep]]
-                if len(label_keys) == 0: continue
+                if len(label_keys) == 0:
+                    continue
                 count_indices = count_indices[keep]
 
                 # Find indices where label_keys are set
-                keep = (label_keys != sentinal).nonzero()[0]
+                keep = (label_keys != sentinel).nonzero()[0]
                 label_keys = label_keys[keep]
-                if len(label_keys) == 0: continue
+                if len(label_keys) == 0:
+                    continue
                 count_indices = count_indices[keep]
 
                 #raw_input()
@@ -620,7 +641,8 @@ def calc_aggregation(segmentation, features, mode, groups, components,
                     counted_features += 1
                     feature_counted = True
 
-        if quick: break
+        if quick:
+            break
 
     return (counts, counted_features)
 
@@ -914,7 +936,8 @@ def parse_options(args):
     group = OptionGroup(parser, "Main aggregation options")
     group.add_option("--mode", choices=MODES,
                      dest="mode", type="choice", default=DEFAULT_MODE,
-                     help="one of: "+str(MODES)+". [default: %default]")
+                     help="one of: %s. [default: %%default]" % \
+                     ", ".join(MODES))
     group.add_option("-f", "--flank-bases", metavar="N",
                      dest="flankbases", type="int", default=FLANK_BASES,
                      help="Aggregate this many base pairs off each"
